$nomod51                ;der Modus fuer 8051 wird abgeschaltet
$debug
$nolist                 ;es wird kein Listing fuer reg535 erstellt
$include(reg535.pdf)    ;die 80535-spezifischen Daten
$list                   ;es wird ein Listing erstellt
$title (KLOPF_W.A51)
;------------------------------------------------------------------------
;Programmbeschreibung
;------------------------------------------------------------------------
;Programm:	KLOPF_W
;Timer2 steuert einen Servomotor mit der Periodendauer von: 20ms
;Stellung: 0  - F7F0h
;Mit Hilfe des Timer2_Interrupts wird die Zeit: DELTA_T2 von 1 s realisiert. 
;Nach einer Sekunde wird eine nop-Operation durchgefhrt.

;
;
;
;Erstellt am: Sonntag, 24. Juni 2007 12:29:41 
;Programmiert: Prof. J. Walter
;
;Verwendete Einspruenge: keine
;
;Verwendete Unterprogramme: keine
;
;
;Verwendete Register und Variable:
;Registerbank(0)
;R2
;
;Kommentar:
;
;Aenderungen:
;Geaendert am: Sonntag, 24. Juni 2007 12:29:41
;
;
;------------------------------------------------------------------------
;Initialisierungsteil fuer allgemeine Konstanten
;------------------------------------------------------------------------
ANZ_INT_T2		EQU	50			;Anzahl der Timer 2 Interrupts 
ANZ_INC			EQU 57			;Anzahl der Inkremente
Periode_low     EQU 0E0H		;legt die Periodendauer von Timer 2 fest 20ms
Periode_high    EQU 0B1H		;45536 = 0B1E0H


UEL EQU	0F0h					;Unterer Endwert
UEH EQU	0F7h
OEL	EQU	080h					;Oberer Endwert
OEH	EQU	0FDh


CSEG AT 0H              ;Legt absolute Codesegmentadresse auf 0h
jmp ABFRAGE
;
;------------------------------------------------------------------------
;Interrupt-Vektoren
;------------------------------------------------------------------------
ORG 02bH			;Timer2 Interrupt
clr TF2				;Interrupt bestaetigen
call ISR_T2			;Interrupt-Service Routine aufrufen
reti

;------------------------------------------------------------------------
;Initialisierungsteil fuer On-Chip Peripherie
;------------------------------------------------------------------------
ORG 100H                		;Programmstart bei 100H
INIT:
								;TIMER2
mov  T2CON,#00010000B   		;Timer2 Einstellung:Timer mit f=f(osz)/12
                        		;Mode 0 ->Reload bei Ueberlauf
mov CCEN,#00001000B    			;Compare Mode des CC1 freigeben

mov CRCL,#Periode_low   		;Low Byte von Compare Reload Capture
mov CRCH,#Periode_high  		;High Byte von Compare Reload Capture

mov CCH1,#UEL					;Pulsweitenmodulation Anfangswert
mov CCL1,#UEH
 								;Interrupt-Freigabe
mov IEN0,#10110000B				;Interrupt Freigabe Timer 2, Serielle Schnittstelle

mov R2,#ANZ_INT_T2
mov R3,#ANZ_INC

setb T2I0						;Timer 2 zaehlt
;------------------------------------------------------------------------
;Programmschleife
;------------------------------------------------------------------------

ABFRAGE:
jnb P1.2,INIT			;Messung startet
jmp ABFRAGE


ISR_T2:
djnz R2,AUS_DER_ISR_T2	;Interrupts T2 zaehlen
DELTA_T2:
mov R2,#ANZ_INT_T2		;R2 neu laden
NOP
cpl P4.0				;FUER KORREKTUR
djnz R3,AUS_DER_ISR_T2	;Gaspedal + oder -
clr T2I0				;Timer 2 stoppen
 

AUS_DER_ISR_T2:
ret

end